<?php

namespace Drupal\wordpress_migrate_ui\Form;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\wordpress_migrate_ui\Wizard\ImportWizard;

/**
 * Simple wizard step form.
 *
 * Designed to run on each content type (posts, pages).
 * $form_state->getTemporaryValue('wizard'); Use CachedValues for state.
 */
class BodyFieldSelectForm extends FormBase {

  /**
   * Constructs a new BodyFieldSelectForm.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   */
  public function __construct(
    protected readonly EntityFieldManagerInterface $entityFieldManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): BodyFieldSelectForm|static {
    return new static(
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'wordpress_migrate_body_field_select_form';
  }

  /**
   * Via $form_state know which content type is here, pick text_with_summary.
   *
   * Use $cached_values for setting this up.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Start clean in case we came here via Previous, then set again.
    $cached_values = $form_state->getTemporaryValue('wizard');
    unset($cached_values['body_field']);
    $form_state->setTemporaryValue('wizard', $cached_values);
    $wp_type = (string) $cached_values['wordpress_content_type'];
    // String post or page. Key off it to set new attribute.
    $field_map = $this->entityFieldManager->getFieldMap();
    $needle_form_el_type = 'text_with_summary';
    $drupal_content_type = (string) $cached_values[$wp_type]['type'];
    $form['overview'] = [
      '#markup' => $this->t('Here you may choose the \'Text (formatted, long, with summary)\' field to import WordPress @type body text into. Your <strong>:drupaltype</strong> content type must have a body field selected below, or
      the new node will not have any body text or content. <br /> If missing, the field
      can be created within <strong>:drupaltype</strong> fields via: Add Field: \'Formatted Text \'. Currently,
       both the main body and the excerpt (summary) will get imported to the  field selected below.', ['@type' => $wp_type, ':drupaltype' => $drupal_content_type]),
    ];
    $options = ['' => $this->t('Do not import')];
    $options = ImportWizard::getBundleFieldnameOptionsFromFieldmap($field_map, $cached_values, $options, $wp_type, $needle_form_el_type);

    $form['body_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Import WordPress @type body text field into:', ['@type' => $wp_type]),
      '#options' => $options,
      '#description' => $this->t('The @type body text will be imported to this field (within the field: full body, and excerpt/summary).', ['@type' => $wp_type]),
    ];

    // Select body automatically.
    if (array_key_exists('body', $options)) {
      $form['body_field']['#default_value'] = $options['body'];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
  }

  /**
   * Uses $cached_values wizard $form_state to set results per content type.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $cached_values = $form_state->getTemporaryValue('wizard');
    $content_type = $cached_values['wordpress_content_type'];
    $cached_values[$content_type]['body_field'] = $form_state->getValue('body_field');
    $form_state->setTemporaryValue('wizard', $cached_values);
  }

}
