<?php

namespace Drupal\wordpress_migrate_ui\Form;

use Drupal\Component\Utility\Environment;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\ByteSizeMarkup;
use Drupal\Core\TypedData\TypedDataManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Simple wizard step form.
 */
class SourceSelectForm extends FormBase {

  /**
   * Typed Data Manager.
   *
   * @var \Drupal\Core\TypedData\TypedDataManager
   */
  public TypedDataManager $typedDataManager;

  public function __construct(TypedDataManager $typed_manager) {
    $this->typedDataManager = $typed_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('typed_data_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'wordpress_migrate_source_select_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // @todo Make sure we have a private directory configured.
    // @link https://www.drupal.org/node/2742291
    // @todo Support importing directly from WP admin interface.
    // @link https://www.drupal.org/node/2742293
    // @todo Support importing from the directory directly.
    // @link https://www.drupal.org/node/2742295
    // @todo Support importing from a database.
    // @link https://www.drupal.org/node/2742299
    $form['overview'] = [
      '#markup' => $this->t('This wizard supports importing into your Drupal site from a WordPress blog. To be able to use this wizard, you must have an XML file exported from the blog.'),
    ];
    $form['description'] = [
      '#markup' => $this->t('<br /><br />You will be led through a series of steps, allowing you to customize what will be imported into Drupal and how it will be mapped. At the end of this process, a migration group will be generated.'),
    ];
    $form['wxr_file'] = [
      '#type' => 'file',
      '#title' => $this->t('WordPress exported file (WXR)'),
      '#description' => $this->t('Select an exported WordPress file (.xml extension). Maximum file size is @size.',
        ['@size' => $this->formatByteSize(Environment::getUploadMaxSize())]),
    ];
    $form['keep_wxr_file'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Permanently save uploaded WXR file'),
      '#description' => $this->t('The uploaded WXR file will be kept in your site permanently. It will always be visible on the "Files" section of the Content administration area.'),
    ];
    $form['base_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base url of the WordPress site'),
      '#description' => $this->t("For example: 'https://test.example.com'. Do not include trailing slashes. This is used for generating new URL paths using the pathauto module."),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $all_files = $this->getRequest()->files->get('files', []);
    if (!isset($all_files['wxr_file'])) {
      $form_state->setErrorByName('wxr_file', $this->t('You must upload a file to continue.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $validators = ['file_validate_extensions' => ['xml']];
    // file_save_upload renames if file already exists (default behavior)
    $file = file_save_upload('wxr_file', $this->getFileValidators(), 'public://', 0);
    if ($file) {
      $cached_values = $form_state->getTemporaryValue('wizard');
      $cached_values['file_uri'] = $file->getFileUri();
      $cached_values['base_url'] = $form_state->getValue('base_url');
      if ($form_state->getValue('keep_wxr_file') === 1) {
        /* Set the status flag permanent of the file object */
        $file->setPermanent();
        /* Save the file in database */
        try {
          $file->save();
        }
        catch (EntityStorageException $e) {
          $this->messenger()->addError('The WXR file failed to upload. Please try again.');
          $this->logger('wordpress_migrate')->error('The WXR file failed to upload. (EntityStorageException)');
        }
      }
      $form_state->setTemporaryValue('wizard', $cached_values);
      // @todo Preprocess the file.
      // @link https://www.drupal.org/node/2742301
    }
    else {
      $form_state->setRebuild();
      $this->messenger()->addError($this->t('WXR file upload failed. Please try again. Your log messages may have additional information.'));
      $this->logger('wordpress_migrate')->error('The WXR file failed to upload.');
    }
  }

  /**
   * Use the appropriate code depending on what's available.
   *
   * @todo Replace when D9 is no longer supported.
   */
  protected function formatByteSize($bytes) {
    if (class_exists('ByteSizeMarkup')) {
      return ByteSizeMarkup::create($bytes);
    }
    elseif (function_exists('format_size')) {
      // @phpstan-ignore-next-line
      return format_size($bytes);
    }
    else {
      return '' . $bytes;
    }
  }

  /**
   * Use the appropriate code depending on what's available.
   *
   * @todo Replace when D9 is no longer supported.
   */
  protected function getFileValidators(): array {
    $constraint_definitions = $this->typedDataManager->getValidationConstraintManager()->getDefinitions();
    if (isset($constraint_definitions['FileExtension'])) {
      $validators = ['FileExtension' => ['extensions' => 'xml']];
    }
    else {
      $validators = ['file_validate_extensions' => ['xml']];
    }

    return $validators;
  }

}
