<?php

namespace Drupal\wordpress_migrate_ui\Wizard;

use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\TempStore\SharedTempStoreFactory;
use Drupal\ctools\Wizard\FormWizardBase;
use Drupal\wordpress_migrate\WordpressMigrationGeneratorFactoryInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Imports the wizard.
 *
 * Inside the form wizard steps:
 * $form_state->getTemporaryValue('wizard'); Use CachedValues for state.
 *
 * @package Drupal\wordpress_migrate_ui\Wizard
 */
class ImportWizard extends FormWizardBase {

  /**
   * The factory for WordPress migration generators.
   *
   * @var \Drupal\wordpress_migrate\WordpressMigrationGeneratorFactoryInterface
   */
  protected WordpressMigrationGeneratorFactoryInterface $wordpressMigrationGeneratorFactory;

  /**
   * Construct import wizard with dependency injections.
   *
   * @param \Drupal\Core\TempStore\SharedTempStoreFactory $tempstore
   *   Tempstore Factory for keeping track of values in each step of the
   *   wizard.
   * @param \Drupal\Core\Form\FormBuilderInterface $builder
   *   The Form Builder.
   * @param \Drupal\Core\DependencyInjection\ClassResolverInterface $class_resolver
   *   The class resolver.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match object.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param string $tempstore_id
   *   The shared temp store factory collection name.
   * @param \Drupal\wordpress_migrate\WordpressMigrationGeneratorFactoryInterface $wordpress_migration_generator_factory
   *   The factory for WordPress migration generators.
   * @param null $machine_name
   *   The SharedTempStore key for our current wizard values.
   * @param null $step
   *   The current active step of the wizard.
   */
  public function __construct(
    SharedTempStoreFactory $tempstore,
    FormBuilderInterface $builder,
    ClassResolverInterface $class_resolver,
    EventDispatcherInterface $event_dispatcher,
    RouteMatchInterface $route_match,
    RendererInterface $renderer,
    string $tempstore_id,
    WordpressMigrationGeneratorFactoryInterface $wordpress_migration_generator_factory,
    $machine_name = NULL,
    $step = NULL,
  ) {
    parent::__construct($tempstore, $builder, $class_resolver, $event_dispatcher, $route_match, $renderer, $tempstore_id, $machine_name, $step);
    $this->wordpressMigrationGeneratorFactory = $wordpress_migration_generator_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getParameters() {
    $parameters = [
      'tempstore' => \Drupal::service('tempstore.shared'),
      'builder' => \Drupal::service('form_builder'),
      'class_resolver' => \Drupal::service('class_resolver'),
      'event_dispatcher' => \Drupal::service('event_dispatcher'),
      'route_match' => \Drupal::service('current_route_match'),
      'renderer' => \Drupal::service('renderer'),
      'wordpress_migration_generator_factory' => \Drupal::service('wordpress_migration_generator_factory'),
    ];
    return $parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function getOperations($cached_values): array {
    $steps = [
      'source_select' => [
        'form' => 'Drupal\wordpress_migrate_ui\Form\SourceSelectForm',
        'title' => $this->t('Data source'),
      ],
      'authors' => [
        'form' => 'Drupal\wordpress_migrate_ui\Form\AuthorForm',
        'title' => $this->t('Authors'),
      ],
      'vocabulary_select' => [
        'form' => 'Drupal\wordpress_migrate_ui\Form\VocabularySelectForm',
        'title' => $this->t('Vocabularies'),
      ],
      'content_select' => [
        'form' => 'Drupal\wordpress_migrate_ui\Form\ContentSelectForm',
        'title' => $this->t('Content'),
      ],
      'image_select' => [
        'form' => 'Drupal\wordpress_migrate_ui\Form\ImageSelectForm',
        'title' => $this->t('Featured images'),
      ],
    ];
    // Dynamically add the content migration(s) that have been configured by
    // ContentSelectForm.
    if (array_key_exists('post', $cached_values) && $cached_values['post']['type'] !== '') {
      $steps += [
        'blog_post' => [
          'form' => 'Drupal\wordpress_migrate_ui\Form\ContentTypeForm',
          'title' => $this->t('Posts'),
          'values' => ['wordpress_content_type' => 'post'],
        ],
        'body_field_select_post' => [
          'form' => 'Drupal\wordpress_migrate_ui\Form\BodyFieldSelectForm',
          'title' => $this->t('Body Field for Posts'),
          'values' => ['wordpress_content_type' => 'post'],
        ],
      ];
    }
    if (array_key_exists('page', $cached_values) && $cached_values['page']['type'] !== '') {
      $steps += [
        'page' => [
          'form' => 'Drupal\wordpress_migrate_ui\Form\ContentTypeForm',
          'title' => $this->t('Pages'),
          'values' => ['wordpress_content_type' => 'page'],
        ],
        'body_field_select_page' => [
          'form' => 'Drupal\wordpress_migrate_ui\Form\BodyFieldSelectForm',
          'title' => $this->t('Body Field for Pages'),
          'values' => ['wordpress_content_type' => 'page'],
        ],
      ];
    }
    $steps += [
      'review' => [
        'form' => 'Drupal\wordpress_migrate_ui\Form\ReviewForm',
        'title' => $this->t('Review'),
        'values' => ['wordpress_content_type' => ''],
      ],
    ];
    return $steps;
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteName(): string {
    return 'wordpress_migrate_ui.wizard.import.step';
  }

  /**
   * {@inheritdoc}
   */
  public function finish(array &$form, FormStateInterface $form_state): void {
    $cached_values = $form_state->getTemporaryValue('wizard');
    $generator = $this->wordpressMigrationGeneratorFactory->createGenerator();
    $generator->createMigrations($cached_values);
    // Go to the dashboard for this migration group.
    $form_state->setRedirect('entity.migration.list', ['migration_group' => $cached_values['group_id']]);
    parent::finish($form, $form_state);
  }

  /**
   * For inside steps, select correct choice options for body text field.
   *
   * Keyed form select elements for the dropdown fields to pick from. Per
   * whichever 'post' or 'page' + target content type picked earlier.
   *
   * @param array $field_map
   * @param array $cached_vals
   * @param $options
   *   array Select menu element.
   * @param $wp_type
   *   string 'post' or 'page'.
   */
  public static function getBundleFieldnameOptionsFromFieldmap(array $field_map, array $cached_vals, array $options, string $wp_type, string $needle_form_el_type): array {
    $drupal_content_type = $cached_vals[$wp_type]['type'];
    foreach ($field_map as $entity_type => $fields) {
      if ($entity_type === 'node') {
        foreach ($fields as $field_name => $field_settings) {
          // The magic.
          if (($field_settings['type'] === $needle_form_el_type) && array_intersect($field_settings['bundles'], [$drupal_content_type])) {
            $options[$field_name] = $field_name;
          }
        }
      }
    }
    return $options;
  }

}
